import 'package:eyflutter_core/dio/beans/application_info.dart';
import 'package:eyflutter_core/event/on_eyflutter_config.dart';
import 'package:eyflutter_core/event/on_scheme_listener.dart';
import 'package:eyflutter_core/kit/parts/config_manager.dart';
import 'package:eyflutter_core/kit/utils/json_utils.dart';
import 'package:eyflutter_core/kit/utils/string/json_extension.dart';
import 'package:eyflutter_core/mq/cloud_channel_manager.dart';
import 'package:eyflutter_core/storage/memory_utils.dart';

mixin OnAddressConfig {
  /// api基础地址
  /// [hostType] 主机类型(根据hostType配置基础地址)
  String baseUrl({required int hostType});

  /// 资源基础url
  /// [resType] 资源类型(根据resType配置资源地址)
  String resBaseUrl({required int resType});
}

class EyFlutter {
  factory EyFlutter() => _getInstance();

  static EyFlutter get instance => _getInstance();
  static EyFlutter? _instance;

  static EyFlutter _getInstance() {
    return _instance ??= EyFlutter._internal();
  }

  EyFlutter._internal();

  String _appInfoKey = "e8f42f0a6a9de326";
  String _startTimeKey = "fba428faac04b4cd";
  String _schemeConfigKey = "02b4309b34ab4316";
  String _addressConfigKey = "0414193a4016a176";
  String _closeDeviceSyncKey = "90e40e0ae575ed5e";
  String _debugKey = "8374a499ebf18619";
  String _platformKey = "f9445f8b02436466";
  String _eyConfigKey = "f0b4595909fb23aa";

  ApplicationInfo _appInfo = ApplicationInfo();

  void setAppInfo(ApplicationInfo? appInfo) {
    if (appInfo != null) {
      _appInfo = appInfo;
    }
  }

  ApplicationInfo get appInfo => _appInfo;

  /// 注册应用配置
  /// OnEyFlutterConfig实现类
  void regAppConfig<T extends OnEyFlutterConfig>(T? config) {
    if (config == null) {
      return;
    }
    ConfigManager.instance.addConfig(_eyConfigKey, config);
  }

  OnEyFlutterConfig? _getEyFlutterConfig() {
    var config = ConfigManager.instance.getConfig(_eyConfigKey);
    if (config is! OnEyFlutterConfig) {
      return null;
    }
    return config;
  }

  /// 获取eyflutter配置项
  OnEyFlutterConfig? get appConfig => _getEyFlutterConfig();

  /// 获取应用信息
  Future<ApplicationInfo> getAppInfo() async {
    var appJsonData = await CloudChannelManager.instance.send<String>(_appInfoKey);
    if (appJsonData.isEmptyJson) {
      return Future.value(ApplicationInfo());
    }
    var map = JsonUtils.fromJson(appJsonData!);
    var applicationInfo = ApplicationInfo.fromJson(map);
    _appInfo = applicationInfo;
    return Future.value(applicationInfo);
  }

  /// 设置应用启动时间
  void setStartTime({required int time}) {
    MemoryUtils.instance.set(_startTimeKey, time);
  }

  /// 获取应用启动时间
  int get startTime => MemoryUtils.instance.getInt(_startTimeKey);

  /// 注册scheme对象
  void registerScheme<T extends OnSchemeListener>(T config) {
    ConfigManager.instance.addConfig(_schemeConfigKey, config);
  }

  OnSchemeListener? _getSchemeConfig() {
    var config = ConfigManager.instance.getConfig(_schemeConfigKey);
    if (config is! OnSchemeListener) {
      return null;
    }
    return config;
  }

  /// 获取scheme配置
  OnSchemeListener? get schemeConfig => _getSchemeConfig();

  /// 注册地址配置
  /// OnAddressConfig实现类
  void regAddressConfig<T extends OnAddressConfig>(T? config) {
    if (config == null) {
      return;
    }
    ConfigManager.instance.addConfig(_addressConfigKey, config);
  }

  OnAddressConfig? _getAddressConfig() {
    var config = ConfigManager.instance.getConfig(_addressConfigKey);
    if (config is! OnAddressConfig) {
      return null;
    }
    return config;
  }

  /// 获取基础地址配置
  OnAddressConfig? get addressConfig => _getAddressConfig();

  /// 是否禁用设备信息上报(默认禁用)
  /// [report] true-在集成eyflutter-cmi插件的前提下启动应用时系统获取设备信息并上报,false-不上传设备信息;
  void disableDeviceInfoReport(bool report) {
    MemoryUtils.instance.set(_closeDeviceSyncKey, report);
  }

  /// 是否禁用设备信息上报(默认禁用)
  bool isDisableDeviceInfoReport() {
    return MemoryUtils.instance.getBool(_closeDeviceSyncKey, defaultValue: true);
  }

  /// 是否禁用同步平台key(默认禁用)
  void disableSyncPlatformKeys(bool disable) {
    MemoryUtils.instance.set(_platformKey, disable);
  }

  /// 是否禁用同步平台key(默认禁用)
  bool isDisableSyncPlatformKeys() {
    return MemoryUtils.instance.getBool(_platformKey, defaultValue: true);
  }

  /// 设置debug模式(默认非debug模式)
  void debugMode(bool debug) {
    MemoryUtils.instance.set(_debugKey, debug);
  }

  /// 是否debug模式(默认非debug模式)
  bool isDebug() {
    return MemoryUtils.instance.getBool(_debugKey, defaultValue: false);
  }

  /// 安装apk
  /// [filePath] apk文件路径
  /// {"flag":"setting"} flag=setting打开安装未知来源设置页面,flag=done安装成功
  Future<String?> installApk(String filePath) {
    return CloudChannelManager.instance.send<String>("d0b49828a514e8b0", arguments: {"path": filePath});
  }
}
