import 'package:eyflutter_core/dio/beans/retrofit.dart';
import 'package:eyflutter_core/dio/dio_manager.dart';
import 'package:eyflutter_core/dio/enums/method.dart';
import 'package:eyflutter_core/ey_flutter.dart';
import 'package:eyflutter_core/kit/parts/config_manager.dart';
import 'package:eyflutter_core/kit/timer/system.dart';
import 'package:eyflutter_core/kit/utils/dynamic_utils.dart';
import 'package:eyflutter_core/kit/utils/json_utils.dart';
import 'package:eyflutter_core/kit/utils/map/map_extension.dart';
import 'package:eyflutter_core/kit/utils/string/json_extension.dart';
import 'package:eyflutter_core/kit/utils/string/string_extension.dart';
import 'package:eyflutter_core/kit/utils/string/url_extension.dart';
import 'package:eyflutter_core/storage/mmkv_utils.dart';
import 'package:flutter/services.dart';

mixin OnConfigTableInterceptor {
  /// 配置表请求地址
  String onUrl();

  /// 配置表响应数据解析(解析后数据格式为key-value对象的json数据)
  /// (一般用于api返回json格式与框架json解析格式不一致时处理)
  String? onResponseParse(String response);
}

class ConfigTable {
  factory ConfigTable() => _getInstance();

  static ConfigTable get instance => _getInstance();
  static ConfigTable? _instance;

  static ConfigTable _getInstance() {
    return _instance ??= new ConfigTable._internal();
  }

  ConfigTable._internal();

  Map<String, dynamic> _tempMap = {};
  String _configCacheKey = "d954677b2383f5f9";
  String _prevSyncTimeKey = "26e4574ac4fa303a";
  String _configTableKey = "87645c397a4ca43e";
  String _appKey = "";
  String? _configData;
  int _prevSyncTime = 0;

  /// 配置表对象注册
  /// OnConfigTableInterceptor实现类
  void register<T extends OnConfigTableInterceptor>(T? config) {
    if (config == null) {
      return;
    }
    ConfigManager.instance.addConfig(_configTableKey, config);
  }

  OnConfigTableInterceptor? _getRegister() {
    var config = ConfigManager.instance.getConfig(_configTableKey);
    if (config is! OnConfigTableInterceptor) {
      return null;
    }
    return config;
  }

  String _getAppKey() {
    if (_appKey.isEmptyString) {
      _appKey = EyFlutter.instance.appConfig?.appKey ?? "";
    }
    return _appKey;
  }

  /// 获取配置值
  /// [configKey] 配置表健
  Future<String> getValue({required String configKey}) {
    if (_tempMap.containsKey(configKey)) {
      var value = _tempMap.getValue<String>(configKey);
      return Future.value(value);
    }
    return _fromCacheTake(configKey);
  }

  /// 获取配置值
  /// [configKey] 配置表健
  Future<int> getDigitValue({required String configKey}) async {
    var value = await this.getValue(configKey: configKey);
    return Future.value(value.toInt);
  }

  /// 获取配置值
  /// [configKey] 配置表健
  Future<bool> getStateValue({required String configKey}) async {
    var value = await this.getValue(configKey: configKey);
    return Future.value(value.isTrue);
  }

  Future<String> _match(String configKey, String configData, bool isFromCache) {
    String matchValue = "";
    Map<String, dynamic> map = JsonUtils.fromJson(configData);
    map.forEach((key, value) {
      if (configKey.isNotEmptyString && (key == configKey)) {
        matchValue = value.trim();
      }
      _tempMap[key] = value;
    });
    if (isFromCache) {
      if (matchValue.isEmptyString) {
        return _fromNetTake(configKey);
      } else {
        return Future.value(matchValue);
      }
    } else {
      return Future.value(matchValue);
    }
  }

  Future<String> _fromCacheTake(String configKey) async {
    if (_configData.isNotEmptyString) {
      return _match(configKey, _configData!, false);
    }
    _configData = await _getCacheConfig();
    if (_configData.isEmptyJson) {
      return _fromNetTake(configKey);
    }
    return _match(configKey, _configData!, true);
  }

  Future<String> _fromNetTake(String configKey) async {
    var tableInterceptor = _getRegister();
    if (tableInterceptor == null) {
      return Future.value("");
    }
    var url = tableInterceptor.onUrl();
    if (!url.isUrl) {
      return Future.value("");
    }
    Retrofit retrofit = Retrofit(api: url, method: Method.GET, cacheKey: "f5e445f9a5096c2c", cacheTime: 90000);
    var response = await DioManager.instance.asyncRequestString(retrofit: retrofit);
    if (response.isEmptyString) {
      return Future.value("");
    }
    var responseParse = tableInterceptor.onResponseParse(response);
    var res = responseParse.isEmptyString ? response : (responseParse ?? "");
    _configData = res;
    MmkvUtils.instance.putString("${_getAppKey()}_${_configCacheKey}", value: res, part: MMkvPart.normal);
    _prevSyncTime = System.currentTimeMillis;
    MmkvUtils.instance.putInt("${_getAppKey()}_${_prevSyncTimeKey}", value: _prevSyncTime, part: MMkvPart.state);
    return _match(configKey, res, false);
  }

  void _accessibilityNetTake(Function() complete) {
    _fromNetTake("").then((netTake) {
      if (_configData.isEmptyString) {
        _accessibilityNetTake(complete);
      } else {
        complete();
      }
    });
  }

  //获取上一次时间
  Future<int> _getPrevTime() async {
    if (_prevSyncTime <= 0) {
      var time = await MmkvUtils.instance.getInt("${_getAppKey()}_${_prevSyncTimeKey}", part: MMkvPart.state);
      _prevSyncTime = time ?? 0;
    }
    return Future.value(_prevSyncTime);
  }

  Future<String?> _getCacheConfig() async {
    if (!_configData.isEmptyJson) {
      return Future.value(_configData);
    }
    var cache = await MmkvUtils.instance.getString("${_getAppKey()}_${_configCacheKey}", part: MMkvPart.normal);
    if (!cache.isEmptyJson) {
      return Future.value(cache);
    }
    return _assetsData;
  }

  Future<String> get _assetsData async {
    var assetPath = "assets/data/config_table.json";
    if (!(await assetPath.hasAsset)) {
      return "{}";
    }
    var loadString = await rootBundle.loadString(assetPath);
    if (!loadString.isEmptyJson) {
      _configData = loadString;
    }
    return Future.value(loadString);
  }

  /// 同步配置表(在应用启动时调用)
  /// [complete] 同步完成
  /// [instantly] 即刻生效(每次获取)
  void syncConfigTable({required Function() complete, bool instantly = false}) {
    if (instantly) {
      _accessibilityNetTake(complete);
    } else {
      _getCacheConfig().then((configCache) {
        Map<String, dynamic> map = JsonUtils.fromJson(configCache ?? "{}");
        map.forEach((key, value) {
          _tempMap[key] = value;
        });
        complete();
      });
    }
    _startSyncConfig();
  }

  void _startSyncConfig() {
    _getPrevTime().then((prevTime) {
      var timeDiff = System.currentTimeMillis - prevTime;
      if (timeDiff >= 14400000) {
        _fromNetTake("");
      }
    });
  }
}
